/* Copyright(c) 2017-2018 Alejandro Sirgo Rica & Contributors
* This file is part of Kylin-Screenshot.

 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "capturelauncher.h"
#include "src/core/controller.h"
#include "src/widgets/imagelabel.h"
#include "src/widgets/notificationwidget.h"
#include "src/utils/screengrabber.h"
#include "src/utils/screenshotsaver.h"
#include "src/widgets/imagelabel.h"
#include <QCheckBox>
#include <QPushButton>
#include <QGridLayout>
#include <QVBoxLayout>
#include <QSpinBox>
#include <QLabel>
#include <QComboBox>
#include <QMimeData>
#include <QDrag>
#include <QFormLayout>
#include <QStandardPaths>
#include "xatomhelper.h"
#define ICON_THEME_NAME           "iconThemeName"
// https://github.com/KDE/spectacle/blob/941c1a517be82bed25d1254ebd735c29b0d2951c/src/Gui/KSWidget.cpp
// https://github.com/KDE/spectacle/blob/941c1a517be82bed25d1254ebd735c29b0d2951c/src/Gui/KSMainWindow.cpp
#define ORG_UKUI_STYLE            "org.ukui.style"

CaptureLauncher::CaptureLauncher(QWidget *parent) :
    QWidget(parent),
    m_id(0)
{
    setAccessibleName("kylin-screenshot_QWidget_captureLauncher");
    const QByteArray style_id(ORG_UKUI_STYLE);
    if(QGSettings::isSchemaInstalled(style_id)){
        m_styleSettings = new QGSettings(style_id);
    }
    setWindowIcon(QIcon::fromTheme("kylin-screenshot"));
    // setWindowTitle(tr("Screenshot"));
    setAttribute(Qt::WA_DeleteOnClose);
    connect(Controller::getInstance(), &Controller::captureTaken,
            this, &CaptureLauncher::captureTaken);
    connect(Controller::getInstance(), &Controller::captureFailed,
            this, &CaptureLauncher::captureFailed);

    MotifWmHints hints;
    hints.flags = MWM_HINTS_FUNCTIONS | MWM_HINTS_DECORATIONS;
    hints.functions = MWM_FUNC_ALL;
    hints.decorations = MWM_DECOR_BORDER;
    XAtomHelper::getInstance()->setWindowMotifHint(winId(), hints);

    vbox_wid = new QVBoxLayout();
    wid_1 = new QWidget();
    wid_1->setAttribute(Qt::WA_TranslucentBackground);
    wid_1->setWindowFlags(Qt::FramelessWindowHint);
    wid_1->setAccessibleName("kylin-screenshot_QWidget_wid1");
    hbox_1 = new QHBoxLayout();
    m_titleIcon_label = new  QLabel();
    m_titleIcon_label->setPixmap(QPixmap(QIcon::fromTheme("kylin-screenshot").pixmap(QSize(24, 24))));
    m_titleIcon_label->setAccessibleName("kylin-screenshot_QLabel_titleIcon");
    //m_titleIcon_label->move(8, 8);
    m_titleName_label = new QLabel();
    m_titleName_label->setText(tr("Screenshot"));
    m_titleName_label->setAlignment(Qt::AlignLeft);
    m_titleName_label->setFixedSize(170, 25);
    m_titleName_label->setAccessibleName("kylin-screenshot_QLabel_titleName");
    //m_titleName_label->move(40, 8);

    m_exit_btn = new QPushButton();
    m_exit_btn->setIcon(QIcon::fromTheme("window-close-symbolic"));
    m_exit_btn->setFixedSize(24, 24);
    m_exit_btn->setIconSize(QSize(16, 16));
    m_exit_btn->setProperty("isWindowButton", 0x2);
    m_exit_btn->setProperty("useIconHighlightEffect", 0x8);
    m_exit_btn->setToolTip(tr("Close"));
    m_exit_btn->setAccessibleName("kylin-screenshot_QPushButton_exit");
    m_exit_btn->setFlat(true);
    //m_exit_btn->move(598, 8);

    m_min_btn = new QPushButton();
    m_min_btn->setIcon(QIcon::fromTheme("window-minimize-symbolic"));
    m_min_btn->setFixedSize(24, 24);
    m_min_btn->setIconSize(QSize(16, 16));
    //m_min_btn->move(566, 8);
    m_min_btn->setProperty("isWindowButton", 0x1);
    m_min_btn->setProperty("useIconHighlightEffect", 0x2);
    m_min_btn->setToolTip(tr("Min"));
    m_min_btn->setAccessibleName("kylin-screenshot_QPushButton_min");
    m_min_btn->setFlat(true);

    hbox_2 = new QHBoxLayout();
    wid_2 = new QWidget();
    wid_2->setAttribute(Qt::WA_TranslucentBackground);
    wid_2->setWindowFlags(Qt::FramelessWindowHint);
    wid_2->setAccessibleName("kylin-screenshot_QWidget_wid2");
    //change by zyb : for BUG 126113 end
    m_imageLabel = new ImageLabel();
    bool ok;
    m_imageLabel->setScreenshot(ScreenGrabber().grabEntireDesktop(ok));
    if (!ok) {
    }
    m_imageLabel->setFixedSize(320, 180);
    m_imageLabel->setAccessibleName("kylin-screenshot_QLabel_imageLabel");

    vbox_capture = new QVBoxLayout();
    wid_capture = new QWidget();
    //wid_capture->setFixedWidth(250);
    wid_capture->setAttribute(Qt::WA_TranslucentBackground);
    wid_capture->setWindowFlags(Qt::FramelessWindowHint);
    wid_capture->setAccessibleName("kylin-screenshot_QWidget_widCapture");

    m_CaptureModeLabel = new QLabel(tr("Capture Mode"));
    m_CaptureModeLabel->setAccessibleName("kylin-screenshot_QLabel_captureMode");

    hbox_area = new QHBoxLayout();
    wid_area = new QWidget();
    wid_area->setAttribute(Qt::WA_TranslucentBackground);
    wid_area->setWindowFlags(Qt::FramelessWindowHint);
    wid_area->setAccessibleName("kylin-screenshot_QWidget_widArea");
    m_areaLabel = new  QLabel();
    m_areaLabel->setText(tr("Area:"));
    m_areaLabel->setAccessibleName("kylin-screenshot_QLabel_areaLabel");
    m_captureType = new QComboBox();
    m_captureType->setFixedHeight(36);
    // TODO remember number
    m_captureType->insertItem(0, tr("Rectangular Region"), CaptureRequest::GRAPHICAL_MODE);
    m_captureType->insertItem(1, tr("Full Screen (All Monitors)"), CaptureRequest::FULLSCREEN_MODE);
    m_captureType->setItemData(0, "kylin-screenshot_QComboBox_rectangularRegion", Qt::AccessibleTextRole);
    m_captureType->setItemData(1, "kylin-screenshot_QComboBox_fullScreen", Qt::AccessibleTextRole);
    m_captureType->setAccessibleName("kylin-screenshot_QComboBox_captureType");

    hbox_delay = new QHBoxLayout();
    wid_delay = new QWidget();
    wid_delay->setAttribute(Qt::WA_TranslucentBackground);
    wid_delay->setWindowFlags(Qt::FramelessWindowHint);
    wid_delay->setAccessibleName("kylin-screenshot_QWidget_widDelay");
    m_delayLabel = new QLabel();
    m_delayLabel->setText(tr("Delay:"));
    m_delayLabel->setAccessibleName("kylin-screenshot_QLabel_delayLabel");

    m_delaySpinBox = new QSpinBox();
    m_delaySpinBox->setSingleStep(1.0);
    m_delaySpinBox->setFixedHeight(36);
    m_delaySpinBox->setAccessibleName("kylin-screenshot_QSpinBox_delaySpinBox");
    //m_delaySpinBox->setMinimumWidth(160);

    m_launchButton = new QPushButton(tr("Take shot"));
    //m_launchButton->setFixedSize(96, 36);
    m_launchButton->setFocus();
    m_launchButton->setProperty("isImportant", true);
    m_launchButton->setAccessibleName("kylin-screenshot_QPushButton_launchButton");

    this->setFixedHeight(260);

    setFont();
    initLayout();
    initConnect();
}

void CaptureLauncher::initLayout()
{
    hbox_1->addWidget(m_titleIcon_label);
    hbox_1->addWidget(m_titleName_label);
    hbox_1->addStretch();
    hbox_1->addWidget(m_min_btn);
    hbox_1->addWidget(m_exit_btn);
    wid_1->setLayout(hbox_1);

    hbox_area->addWidget(m_areaLabel);
    hbox_area->addWidget(m_captureType);
    hbox_area->setMargin(0);
    wid_area->setLayout(hbox_area);

    hbox_delay->addWidget(m_delayLabel);
    hbox_delay->addWidget(m_delaySpinBox);
    hbox_delay->setMargin(0);
    wid_delay->setLayout(hbox_delay);

    vbox_capture->addWidget(m_CaptureModeLabel, 0, Qt::AlignLeft);
    vbox_capture->addWidget(wid_area);
    vbox_capture->addWidget(wid_delay);
    vbox_capture->setSpacing(20);
    vbox_capture->addWidget(m_launchButton, 0, Qt::AlignRight);
    vbox_capture->setContentsMargins(0, 0, 0, 0);
    wid_capture->setLayout(vbox_capture);

    hbox_2->addWidget(m_imageLabel, 0, Qt::AlignLeft);
    hbox_2->setSpacing(20);
    hbox_2->addWidget(wid_capture, 0, Qt::AlignRight);
    hbox_2->setContentsMargins(18, 0, 18, 0);
    wid_2->setLayout(hbox_2);

    vbox_wid->addWidget(wid_1);
    vbox_wid->setSpacing(0);
    vbox_wid->addWidget(wid_2);
    vbox_wid->setContentsMargins(2, 0, 2, 20);

    setLayout(vbox_wid);
}

void CaptureLauncher::initConnect()
{
    connect(m_exit_btn, &QPushButton::clicked, this, &CaptureLauncher::close);
    //change by zyb : for BUG 126113
    connect(m_min_btn, &QPushButton::clicked, this, &CaptureLauncher::showMinimized);

    connect(m_imageLabel, &ImageLabel::dragInitiated, this, &CaptureLauncher::startDrag);
    connect(m_delaySpinBox, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged), this, [this](int val) {
        QString sufix = val == 1 ? tr(" second") : tr(" seconds");
        this->m_delaySpinBox->setSuffix(sufix);
    });
    connect(m_launchButton, &QPushButton::pressed, this, &CaptureLauncher::startCapture);

    connect(m_styleSettings,&QGSettings::changed,this,[=](const QString &key) {
        if (key == "systemFontSize" || key == "systemFont") {
            setFont();
       }
    });
    connect(m_styleSettings,&QGSettings::changed,this,[=](const QString &key) {
       if(key==ICON_THEME_NAME){
           setWindowIcon(QIcon::fromTheme("kylin-screenshot"));
           m_titleIcon_label->setPixmap(QPixmap(QIcon::fromTheme("kylin-screenshot").pixmap(QSize(24,24))));
       }
    });
}

// HACK: https://github.com/KDE/spectacle/blob/fa1e780b8bf3df3ac36c410b9ece4ace041f401b/src/Gui/KSMainWindow.cpp#L70
void CaptureLauncher::startCapture()
{
    auto mode = static_cast<CaptureRequest::CaptureMode>(
        m_captureType->currentData().toInt());
    if(mode==CaptureRequest::FULLSCREEN_MODE){
        hide();
    }
    if(mode==CaptureRequest::GRAPHICAL_MODE){
        close();
    }
    CaptureRequest req(mode, 1000 + m_delaySpinBox->value() * 1000);
    m_id = req.id();
    Controller::getInstance()->requestCapture(req);
}

void CaptureLauncher::startDrag()
{
    QDrag *dragHandler = new QDrag(this);
    QMimeData *mimeData = new QMimeData;
    mimeData->setImageData(m_imageLabel->pixmap());
    dragHandler->setMimeData(mimeData);

    dragHandler->setPixmap(m_imageLabel->pixmap()
                           ->scaled(256, 256, Qt::KeepAspectRatioByExpanding,
                                    Qt::SmoothTransformation));
    dragHandler->exec();
}

void CaptureLauncher::captureTaken(uint id, QPixmap p)
{
    if (id == m_id) {
        m_id = 0;
        m_imageLabel->setScreenshot(p);
        auto mode = static_cast<CaptureRequest::CaptureMode>(
            m_captureType->currentData().toInt());
        QStringList a = QStandardPaths::standardLocations(QStandardPaths::PicturesLocation);
        if (mode == CaptureRequest::FULLSCREEN_MODE) {
            ScreenshotSaver().saveToFilesystem(p, a.at(0), ".png");
        }
//        show();
    }
}

void CaptureLauncher::captureFailed(uint id)
{
    if (id == m_id) {
        m_id = 0;
//        show();
    }
}

void CaptureLauncher::setFont()
{
    QString fontFamily = m_styleSettings->get("systemFont").toString();
    double fontSize = m_styleSettings->get("systemFontSize").toDouble();
    QFont f(fontFamily);
    f.setPointSizeF(fontSize + 2);
    m_CaptureModeLabel->setFont(f);
    f.setPointSizeF(fontSize);
    m_titleName_label->setFont(f);
    m_areaLabel->setFont(f);
    m_delayLabel->setFont(f);
    m_launchButton->setFont(f);
    m_delaySpinBox->setFont(f);
    m_captureType->setFont(f);

    QFontMetrics fm_area(m_areaLabel->font());
    int areaWidth = fm_area.width(m_areaLabel->text());
    QFontMetrics fm_delay(m_delayLabel->font());
    int delayWidth = fm_delay.width(m_delayLabel->text());
    int width = areaWidth > delayWidth ? areaWidth : delayWidth;
    m_areaLabel->setFixedWidth(width);
    m_delayLabel->setFixedWidth(width);
    int captureTypeMinWidth = 250 - width;

    QFontMetrics fm_launch(m_launchButton->font());
    int launchWidth = fm_launch.width(m_launchButton->text());
    m_launchButton->setFixedWidth(launchWidth + 30);

    QFontMetrics fm_captureType(m_captureType->font());
    int captureTypeWidth = 0;
    int count = m_captureType->count();
    for (int i = 0; i < count; i++) {
        int textWidth = fm_captureType.width(m_captureType->itemText(i));
        if (textWidth > captureTypeWidth) {
            captureTypeWidth = textWidth;
        }
    }
    m_captureType->setFixedWidth(captureTypeWidth + 40);
    m_captureType->setMinimumWidth(captureTypeMinWidth);
    this->setFixedWidth(m_captureType->width() + width + 400);
    this->setMinimumWidth(630);
    qDebug()<<this->width();
}
